@@ -1,16 +1,9 @@
-use pdf_writer::{
-    ActionType, AnnotationType, BorderType, Content, Name, PdfWriter, Rect, Ref, Str,
-    TextStr,
-};
+use pdf_writer::types::{ActionType, AnnotationType, BorderType};
+use pdf_writer::{Content, Finish, Name, PdfWriter, Rect, Ref, Str, TextStr};
 
 fn main() -> std::io::Result<()> {
-    // Start writing with PDF version 1.7 header. The version is not
-    // semantically important to the writer, but must be present in the output
-    // document.
-    let mut writer = PdfWriter::new(1, 7);
-
-    // Make the output more readable by indenting things with 2 spaces.
-    writer.set_indent(2);
+    // Start writing.
+    let mut writer = PdfWriter::new();
 
     // Define some indirect reference ids we'll use.
     let catalog_id = Ref::new(1);
@@ -24,7 +17,7 @@ fn main() -> std::io::Result<()> {
     writer.catalog(catalog_id).pages(page_tree_id);
 
     // Write the page tree with a single child page.
-    writer.pages(page_tree_id).kids([page_id]);
+    writer.pages(page_tree_id).kids([page_id]).count(1);
 
     // Write a page.
     let mut page = writer.page(page_id);
@@ -38,11 +31,9 @@ fn main() -> std::io::Result<()> {
     // We also create the annotations list here that allows us to have things
     // like links or comments on the page.
     let mut annotations = page.annotations();
-
-    // Write a new annotation.
     let mut annotation = annotations.push();
 
-    // Write the type, area, alt-text, and color for this annotation.
+    // Write the type, area, alt-text, and color for our link annotation.
     annotation.subtype(AnnotationType::Link);
     annotation.rect(Rect::new(215.0, 730.0, 251.0, 748.0));
     annotation.contents(TextStr("Link to the Rust project web page"));
@@ -59,18 +50,18 @@ fn main() -> std::io::Result<()> {
     // Set border and style for the link annotation.
     annotation.border_style().width(2.0).style(BorderType::Underline);
 
-    // We have to drop all the writers that page depends on in order here
-    // because otherwise it would be mutably borrowed until the end of the
-    // block.
-    drop(annotation);
-    drop(annotations);
+    // We have to finish all the writers that depend on the page here because
+    // otherwise they would be mutably borrowed until the end of the block.
+    // Finishing is handled through the writer's `Drop` implementations, so that
+    // you cannot accidentally forget it. The `finish()` method from the `Finish`
+    // trait is just a postfix-style version of dropping.
+    annotation.finish();
+    annotations.finish();
 
     // We also need to specify which resources the page needs, which in our case
     // is only a font that we name "F1" (the specific name doesn't matter).
     page.resources().fonts().pair(font_name, font_id);
-
-    // We also need to drop the page, as with the annotations above.
-    drop(page);
+    page.finish();
 
     // Specify the font we want to use. Because Helvetica is one of the 14 base
     // fonts shipped with every PDF reader, we don't have to embed any font
@@ -86,16 +77,17 @@ fn main() -> std::io::Result<()> {
     // characters.
     let mut content = Content::new();
     content
-        .text()
-        .font(font_name, 14.0)
+        .begin_text()
+        .set_font(font_name, 14.0)
         .next_line(108.0, 734.0)
-        .show(Str(b"Hello World from Rust!"));
+        .show(Str(b"Hello World from Rust!"))
+        .end_text();
 
     writer.stream(text_id, &content.finish());
 
     // Finish writing (this automatically creates the cross-reference table and
     // file trailer) and retrieve the resulting byte buffer.
-    let buf: Vec<u8> = writer.finish(catalog_id);
+    let buf: Vec<u8> = writer.finish();
 
     // Write the thing to a file.
     std::fs::write("target/hello.pdf", buf)
